@@ -7,46 +7,30 @@ use warnings;
 use base qw(Template::Recall::Base);
 
 # This version: only single file template or template string
-our $VERSION='0.18';
+our $VERSION='0.19';
 
 
 sub new {
 
 	my $class = shift;
 	my $self = {};
+	bless $self, $class;
 
 	my ( %h ) = @_;
 
-	# Set default values
-	$self->{'is_file_template'} = 0;
-	$self->{'template_secpat'} = qr/\[\s*=+\s*\w+\s*=+\s*\]/;		# Section pattern
-	$self->{'secpat_delims'} = [ '\[\s*=+\s*', '\s*=+\s*\]' ];	# Section delimiters
-	$self->{'val_delims'} = [ '\[\'', '\'\]' ];
+	# Default values
+	$self->{'secpat'} = qr/\[\s*=+\s*(\w+)\s*=+\s*\]/;
+	$self->{'val_delims'} = [ qr/\['/, qr/'\]/ ];
 	$self->{'trim'} = undef;		# undef=off
-    #TODO remove?
-	$self->{'stored_secs'} = {};	# Store rendered sections internally
 
 
-	bless( $self, $class );
+    if (exists $h{'secpat'} and ref $h{'secpat'}) {
+        $self->{'secpat'} = $h{'secpat'};
+    }
 
-	# User defines section pattern
-	$self->{'template_secpat'} = qr/$h{'secpat'}/ if defined( $h{'secpat'} );
-
-	# Section: User sets 'no delimiters'
-	$self->{'secpat_delims'} = undef
-		if defined($h{'secpat_delims'}) and !ref($h{'secpat_delims'});
-
-	# Section: User specifies delimiters
-	$self->{'secpat_delims'} = [ @{ $h{'secpat_delims'} } ]
-		if defined($h{'secpat_delims'}) and ref($h{'secpat_delims'});
-
-
-	# User sets 'no delimiters'
-	$self->{'val_delims'} = undef if defined($h{'val_delims'}) and !ref($h{'val_delims'});
-
-	# User specifies delimiters
-	$self->{'val_delims'} = [ @{ $h{'val_delims'} } ]
-		if defined($h{'val_delims'}) and ref($h{'val_delims'});
+    if (exists $h{'val_delims'} and ref $h{'val_delims'} eq 'ARRAY') {
+        $self->{'val_delims'} = $h{'val_delims'};
+    }
 
 	# User supplied the template from a string
 
@@ -54,121 +38,95 @@ sub new {
 		$self->init_template($h{'template_str'});
 		return $self;
 	}
-    else {
-        die if ( not defined($h{'template_path'}) or !-e $h{'template_path'} );
-    }
 
-    $self->{'template_path'} = $h{'template_path'};
-    $self->init_template_from_file();
+    die 'Path to template not defined or missing'
+        unless defined($h{'template_path'}) and -e $h{'template_path'};
+
+
+    $self->init_template_from_file($h{'template_path'});
 
 	return $self;
 
 } # new()
 
 
+sub init_template_from_file {
 
+	my ($self, $tpath) = @_;
 
+	my $s;
+    open my $fh, $tpath or die "Couldn't open $tpath: $!";
+	while(<$fh>) { $s .= $_; }
+	close $fh;
+    $self->init_template($s);
+
+}
+
+# Handle template passed by user as string
+sub init_template {
+
+    my ($self, $template) = @_;
+
+    my $sec = [ split( /($self->{'secpat'})/, $template ) ];
+
+    my %h;
+    my $curr = '';
+
+    # Top-down + only one 'body' follows section, why this parse hack works
+    for (my $i=0; $i <= $#{$sec} ; $i++) {
+        my $el = $$sec[$i];
+        next if $el =~ /^$/;
+        if ($el =~ /$self->{'secpat'}/) {
+            $curr = $1;
+            $h{$curr} = '';
+            $i++; # Skip next, it's the section name (an artifact)
+        }
+        else {
+            $h{$curr} = $el;
+        }
+    }
+
+    $self->{'template_secs'} = \%h;
+
+} # init_template()
 
 
 sub render {
 
 	my ( $self, $section, $hash_ref ) = @_;
 
-	return "Error: no section to render: $section\n" if !defined($section);
+	die "Error: no section to render: $section\n" if !defined($section);
 
-    return if (!exists $self->{'template_secs'}->{$section});
+    return if !exists $self->{'template_secs'}->{$section};
 
-    my $sectemp = $self->{'template_secs'}->{$section};
-
-    #TODO is it faster if the default delim allows for *no* spaces between ID
-    return $self->SUPER::render( $sectemp, $hash_ref, $self->{'val_delims'} );
+    return $self->SUPER::render(
+        $self->{'template_secs'}->{$section}, $hash_ref, $self->{'val_delims'});
 
 } # render()
 
 
-
-
-
-
-# Load the single file template into array of sections
-
-sub init_template_from_file {
-
-	my $self = shift;
-
-	my $t;
-    open my $fh, $self->{'template_path'} or die "Couldn't open $self->{'template_path'} $!";
-	while(<$fh>) { $t .= $_; }
-	close $fh;
-    $self->init_template($t);
-
-} # init_file_template()
-
-
-
-
-
-# Handle template passed by user as string
-
-sub init_template {
-
-    my ($self, $template) = @_;
-
-    my $sec = [ split( /($self->{'template_secpat'})/, $template ) ];
-
-    my %h;
-    my $curr = '';
-
-    # top-down + only one 'body' follows section, why this parse hack works
-    for (@$sec) {
-        next if /^$/;
-        if (/$self->{'template_secpat'}/) {
-            $curr = $_;
-            $curr =~ s/$self->{'secpat_delims'}[0]|$self->{'secpat_delims'}[1]//g;
-            $h{$curr} = '';
-        }
-        else {
-            $h{$curr} = $_;
-        }
-    }
-
-    $self->{'template_secs'} = \%h;
-
-}
-
-
-
-
-
-
 # Set trim flags
 sub trim {
 	my ($self, $flag) = @_;
 
-
-
 	# trim() with no params defaults to trimming both ends
-	if (!defined($flag)) {
+	if (!defined $flag) {
 		$self->{'trim'} = 'both';
 		return;
 	}
 
-
-
 	# Turn trimming off
 	if ($flag =~ /^(off|o)$/i) {
 		$self->{'trim'} = undef;
 		return;
 	}
 
-
 	# Make sure we get something valid
 	if ($flag !~ /^(off|left|right|both|l|r|b|o)$/i) {
 		$self->{'trim'} = undef;
 		return;
 	}
 
-
 	$self->{'trim'} = $flag;
 	return;
 
@@ -192,7 +150,7 @@ Template::Recall - "Reverse callback" templating system
 	use Template::Recall;
 
 	# Load template sections from file
-	my $tr = Template::Recall->new( template_path => '/path/to/template_file.html' );
+	my $tr = Template::Recall->new( template_path => '/path/to/template_file.htmt' );
 
 	my @prods = (
 		'soda,sugary goodness,$.99',
@@ -200,7 +158,7 @@ Template::Recall - "Reverse callback" templating system
 		'green tea,wholesome goodness,$1.59'
 		);
 
-	$tr->render('header');
+	print $tr->render('header');
 
 	for (@prods)
 	{
@@ -225,9 +183,9 @@ where needed. Template::Recall works in reverse. Rather than inserting code
 inside the template, the template remains separate, but broken into sections.
 The sections are called from within the code at the appropriate times.
 
-A template section is merely a file on disk. For instance, 'prodrow' above
-(actually F<prodrow.html> in the template directory), might look like
+A template section, such as C<prodrow> above, looks something like
 
+    [=== prodrow ===]
 	<tr>
 		<td>[' product ']</td>
 		<td>[' description ']</td>
@@ -244,13 +202,10 @@ replace, and pass a reference of it along with the template section, i.e.
 
 =head3 C<new( [ template_path =E<gt> $path, secpat =E<gt> $section_pattern, delims =E<gt> ['opening', 'closing'] ] )>
 
-Instantiates the object. If you do not specify C<template_path>, it will assume
-templates are in the directory that the script lives in. If C<template_path>
-points to a file rather than a directory, it loads all the template sections
-from this file. The file must be sectioned using the "section pattern", which
-can be adjusted via the C<secpat> parameter.
+Instantiates the object. You must pass either C<template_path> or
+C<template_str> as the first parameter.
 
-C<secpat>, by default, is C<[\s*=+\s*\w+\s*=+\s*]/>. So if you put all your
+C<secpat>, by default, is C<qr/[\s*=+\s*(\w+)\s*=+\s*]/>. So if you put all your
 template sections in one file, the way Template::Recall knows where to get the
 sections is via this pattern, e.g.
 
@@ -275,13 +230,9 @@ sections is via this pattern, e.g.
 	</body>
 	</html>
 
-You may set C<secpat> to any pattern you wish. Note that if you use delimiters
-(i.e. opening and closing symbols) for the section pattern, you will also need
-to set the C<secpat_delims> parameter to those delimiters. So if you had set
-C<secpat> to that above, you would need also need to set C<secpat_delims =E<gt>
-[ '[\s*=+\s*', '\s*=+\s*]' ]>. If you decide to not use delimiters, and use
-something like C<secpat =E<gt> qr/MYTEMPLATE_SECTION_\w+/>, then you must set
-C<secpat_delims =E<gt> 'no'>.
+You may set C<secpat> to any pattern you wish, but you probably shouldn't
+unless you have some legitimate reason to do so. It needs to be a compiled
+regex too: C<qr//>.
 
 The default delimeters for variables in Template::Recall are C<['> (opening)
 and C<']> (closing). This tells Template::Recall that C<[' price ']> is
@@ -308,10 +259,8 @@ You must specify C<$section>, which tells C<render()> what template
 "section" to load. C<$reference_to_hash> is optional. Sometimes you just want
 to return a template section without any variables. Usually,
 C<$reference_to_hash> will be used, and C<render()> iterates through the hash,
-replacing the F<key> found in the template with the F<value> associated to
-F<key>. A reference was chosen for efficiency. The hash may be large, so either
-pass it using a backslash like in the synopsis, or do something like
-C<$hash_ref = { 'name' =E<gt> 'value' }> and pass C<$hash_ref>.
+replacing the F<key> found in the template with the F<value> associated with 
+F<key>. A reference was chosen for efficiency.
 
 =head3 C<trim( 'off|left|right|both' );>
 
@@ -340,4 +289,8 @@ James Robson E<lt>arbingersys F<AT> gmail F<DOT> comE<gt>
 
 =head1 SEE ALSO
 
-http://perl.apache.org/docs/tutorials/tmpl/comparison/comparison.html
+Some context -- L<http://perl.apache.org/docs/tutorials/tmpl/comparison/comparison.html>
+
+Tutorial -- L<http://www.perl.com/pub/2008/03/14/reverse-callback-templating.html>
+
+Performance comparison -- L<http://soundly.me/template-recall-vs-template-toolkit>
